@@ -1,34 +1,18 @@
 """
 General helpers required for `tqdm.std`.
 """
-from functools import wraps
-from warnings import warn
 import os
 import re
-import subprocess
 import sys
+from ast import literal_eval as safe_eval
+from functools import partial, partialmethod, wraps
+from inspect import signature
+# TODO consider using wcswidth third-party package for 0-width characters
+from unicodedata import east_asian_width
+from warnings import warn
+from weakref import proxy
 
-# py2/3 compat
-try:
-    _range = xrange
-except NameError:
-    _range = range
-
-try:
-    _unich = unichr
-except NameError:
-    _unich = chr
-
-try:
-    _unicode = unicode
-except NameError:
-    _unicode = str
-
-try:
-    _basestring = basestring
-except NameError:
-    _basestring = str
-
+_range, _unich, _unicode, _basestring = range, chr, str, str
 CUR_OS = sys.platform
 IS_WIN = any(CUR_OS.startswith(i) for i in ['win32', 'cygwin'])
 IS_NIX = any(CUR_OS.startswith(i) for i in ['aix', 'linux', 'darwin'])
@@ -48,12 +32,68 @@ else:
         colorama.init()
 
 
+def envwrap(prefix, case_sensitive=False, literal_eval=False, is_method=False):
+    """
+    Override parameter defaults via `os.environ[prefix + param_name]`.
+    Precedence (highest first):
+    - call (`foo(a=3)`)
+    - environ (`FOO_A=2`)
+    - signature (`def foo(a=1)`)
+
+    Parameters
+    ----------
+    prefix  : str
+        Env var prefix, e.g. "FOO_"
+    case_sensitive  : bool, optional
+        If (default: False), treat env var "FOO_Some_ARG" as "FOO_some_arg".
+    literal_eval  : bool, optional
+        Whether to `ast.literal_eval` the detected env var overrides.
+        Otherwise if (default: False), infer types from function signature.
+    is_method  : bool, optional
+        Whether to use `functools.partialmethod`. If (default: False) use `functools.partial`.
+
+    Examples
+    --------
+    ```
+    $ cat foo.py
+    from tqdm.utils import envwrap
+    @envwrap("FOO_")
+    def test(a=1, b=2, c=3):
+        print(f"received: a={a}, b={b}, c={c}")
+
+    $ FOO_A=42 FOO_C=1337 python -c 'import foo; foo.test(c=99)'
+    received: a=42, b=2, c=99
+    ```
+    """
+    i = len(prefix)
+    env_overrides = {k[i:] if case_sensitive else k[i:].lower(): v
+                     for k, v in os.environ.items() if k.startswith(prefix)}
+    part = partialmethod if is_method else partial
+
+    def wrap(func):
+        params = signature(func).parameters
+        overrides = {k: v for k, v in env_overrides.items() if k in params}
+        if literal_eval:
+            return part(func, **{k: safe_eval(v) for k, v in overrides.items()})
+        # use `func` signature to infer env override `type` (fallback to `str`)
+        for k in overrides:
+            param = params[k]
+            if param.annotation is not param.empty:
+                typ = param.annotation
+                # TODO: parse type in {Union, Any, Optional, ...}
+            else:
+                typ = str if param.default is None else type(param.default)
+            overrides[k] = typ(overrides[k])
+        return part(func, **overrides)
+    return wrap
+
+
 class FormatReplace(object):
     """
     >>> a = FormatReplace('something')
     >>> "{:5d}".format(a)
     'something'
-    """
+    """  # NOQA: P102
     def __init__(self, replace=''):
         self.replace = replace
         self.format_called = 0
@@ -138,17 +178,25 @@ class DisableOnWriteError(ObjectWrapper):
         """
         Quietly set `tqdm_instance.miniters=inf` if `func` raises `errno=5`.
         """
+        tqdm_instance = proxy(tqdm_instance)
+
         def inner(*args, **kwargs):
             try:
                 return func(*args, **kwargs)
-            except (IOError, OSError) as e:
+            except OSError as e:
                 if e.errno != 5:
                     raise
-                tqdm_instance.miniters = float('inf')
+                try:
+                    tqdm_instance.miniters = float('inf')
+                except ReferenceError:
+                    pass
             except ValueError as e:
                 if 'closed' not in str(e):
                     raise
-                tqdm_instance.miniters = float('inf')
+                try:
+                    tqdm_instance.miniters = float('inf')
+                except ReferenceError:
+                    pass
         return inner
 
     def __init__(self, wrapped, tqdm_instance):
@@ -198,7 +246,7 @@ def _is_utf(encoding):
     except Exception:
         try:
             return encoding.lower().startswith('utf-') or ('U8' == encoding)
-        except:
+        except Exception:
             return False
     else:
         return True
@@ -237,8 +285,8 @@ def _screen_shape_wrapper():  # pragma: no cover
 
 def _screen_shape_windows(fp):  # pragma: no cover
     try:
-        from ctypes import windll, create_string_buffer
         import struct
+        from ctypes import create_string_buffer, windll
         from sys import stdin, stdout
 
         io_handle = -12  # assume stderr
@@ -254,7 +302,7 @@ def _screen_shape_windows(fp):  # pragma: no cover
             (_bufx, _bufy, _curx, _cury, _wattr, left, top, right, bottom,
              _maxx, _maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
             return right - left, bottom - top  # +1
-    except:
+    except Exception:  # nosec
         pass
     return None, None
 
@@ -263,9 +311,10 @@ def _screen_shape_tput(*_):  # pragma: no cover
     """cygwin xterm (windows)"""
     try:
         import shlex
-        return [int(subprocess.check_call(shlex.split('tput ' + i))) - 1
+        from subprocess import check_call  # nosec
+        return [int(check_call(shlex.split('tput ' + i))) - 1
                 for i in ('cols', 'lines')]
-    except:
+    except Exception:  # nosec
         pass
     return None, None
 
@@ -273,19 +322,19 @@ def _screen_shape_tput(*_):  # pragma: no cover
 def _screen_shape_linux(fp):  # pragma: no cover
 
     try:
-        from termios import TIOCGWINSZ
-        from fcntl import ioctl
         from array import array
+        from fcntl import ioctl
+        from termios import TIOCGWINSZ
     except ImportError:
-        return None
+        return None, None
     else:
         try:
             rows, cols = array('h', ioctl(fp, TIOCGWINSZ, '\0' * 8))[:2]
             return cols, rows
-        except:
+        except Exception:
             try:
                 return [int(os.environ[i]) - 1 for i in ("COLUMNS", "LINES")]
-            except KeyError:
+            except (KeyError, ValueError):
                 return None, None
 
 
@@ -311,14 +360,8 @@ def _term_move_up():  # pragma: no cover
     return '' if (os.name == 'nt') and (colorama is None) else '\x1b[A'
 
 
-try:
-    # TODO consider using wcswidth third-party package for 0-width characters
-    from unicodedata import east_asian_width
-except ImportError:
-    _text_width = len
-else:
-    def _text_width(s):
-        return sum(2 if east_asian_width(ch) in 'FW' else 1 for ch in _unicode(s))
+def _text_width(s):
+    return sum(2 if east_asian_width(ch) in 'FW' else 1 for ch in str(s))
 
 
 def disp_len(data):
